"""
Class for converting tcx to gpx
"""
import logging
from datetime import datetime
from pathlib import Path

from gpxpy import gpx
from tcxparser import TCXParser
import dateutil.parser

# pylint: disable=logging-format-interpolation


LOGGER = logging.getLogger('tcx2gpx')


class TCX2GPX():
    """
    Convert tcx files to gpx.
    """

    def __init__(self, tcx_path, outdir=None):
        """
        Initialise the class.
        """
        self.tcx_path = Path(tcx_path)
        self.outdir = outdir
        self.tcx = None
        self.track_points = None
        self.gpx = gpx.GPX()

    def convert(self):
        """
        Convert tcx to gpx.
        """
        self.read_tcx()
        self.extract_track_points()
        self.create_gpx()
        self.write_gpx()

    def read_tcx(self):
        """
        Read a TCX file.

        Parameter
        ---------
        tcx_path: str
            Valid path to a TCX file.
        """
        try:
            self.tcx = TCXParser(str(self.tcx_path.resolve()))
            LOGGER.info(
                'Reading                     : {}'.format(self.tcx_path))
        except TypeError as not_pathlib:
            print(not_pathlib)
            raise TypeError('File path did not resolve.')

    def extract_track_points(self):
        """
        Extract and combine features from tcx
        """
        self.track_points = zip(self.tcx.position_values(),
                                self.tcx.altitude_points(),
                                self.tcx.time_values())
        LOGGER.info('Extracting track points from : {}'.format(self.tcx_path))

    def create_gpx(self):
        """
        Create GPX object.
        """
        self.gpx.name = dateutil.parser.parse(
            self.tcx.started_at).strftime('%Y-%m-%d %H:%M:%S')
        self.gpx.description = ''
        gpx_track = gpx.GPXTrack(name=dateutil.parser.parse(self.tcx.started_at).strftime('%Y-%m-%d %H:%M:%S'),
                                 description='')
        gpx_track.type = self.tcx.activity_type
        # gpx_track.extensions = '<topografix:color>c0c0c0</topografix:color>'
        self.gpx.tracks.append(gpx_track)
        gpx_segment = gpx.GPXTrackSegment()
        gpx_track.segments.append(gpx_segment)
        for track_point in self.track_points:
            gpx_trackpoint = gpx.GPXTrackPoint(latitude=track_point[0][0],
                                               longitude=track_point[0][1],
                                               elevation=track_point[1],
                                               time=datetime.strptime(track_point[2],
                                                                      '%Y-%m-%dT%H:%M:%SZ'))
            gpx_segment.points.append(gpx_trackpoint)
        LOGGER.info('Creating GPX for             : {}'.format(self.tcx_path))

    def write_gpx(self):
        """
        Write GPX object to file.
        """
        out = str(self.tcx_path.resolve()).replace('.tcx', '.gpx')
        with open(out, 'w') as output:
            output.write(self.gpx.to_xml())
        LOGGER.info('GPX written to               : {}'.format(out))



"""
Fixtures for test_tcx2gpx
"""
from pathlib import Path
import pytest

from tcx2gpx.tcx2gpx import TCX2GPX

TCX_DIR = Path(__file__).resolve().parents[1]


TCX_FILE = TCX_DIR / 'resources' / '2019-10-20 12:51:21.0.tcx'
GPX_FILE = TCX_DIR / 'resources' / '2019-10-20 12:51:21.0.gpx'


@pytest.fixture
def tcx_file():
    """
    Fixture of TCX file
    """
    return TCX2GPX(TCX_FILE)



"""
Batch convert tcx to gpx files.
"""
import logging
from pathlib import Path
import argparse as arg
from multiprocessing import Pool

from tcx2gpx.tcx2gpx import TCX2GPX

# pylint: disable=logging-format-interpolation

LOGGER = logging.getLogger('batch tcx2gpx')


def create_parser():
    """
    Parse arguments for smooth scoring
    """

    parser = arg.ArgumentParser()
    parser.add_argument('-d', '--directory',
                        default='.',
                        help='Directory containing tcx files for conversion',
                        required=False)
    parser.add_argument('-j', '--cores',
                        default='4',
                        help='Number of processors to use.',
                        required=False)
    return parser.parse_args()


def process_tcx(file_path):
    """
    Wrapper to convert individual tcx files to gpx.
    """
    to_convert = TCX2GPX(file_path)
    to_convert.convert()


if __name__ == '__main__':

    PARSER = create_parser()
    TCX_DIR = Path(PARSER.directory)
    LOGGER.info('Searching for files in          : {}'.format(TCX_DIR))
    TCX_FILES = sorted(TCX_DIR.glob('*.tcx'))

    LOGGER.info('Found {} files, processing...'.format(len(TCX_FILES)))
    POOL = Pool(int(PARSER.cores))
    POOL.map(process_tcx, TCX_FILES)



"""
Test tcx2gpy.
"""
from pathlib import Path

from tcxparser import TCXParser
from gpxpy.gpx import GPX

TCX_DIR = Path(__file__).resolve().parents[1]
GPX_FILE = TCX_DIR / 'resources' / '2019-10-20 12:51:21.0.gpx'


def test_tcx2gpx_init(tcx_file):
    """
    Test instantiation results in the correct object type.
    """
    assert isinstance(tcx_file.tcx_path, Path)
    assert isinstance(tcx_file.gpx, GPX)


def test_read_tcx(tcx_file):
    """
    Test reading of TCX
    """
    tcx_file.read_tcx()

    assert isinstance(tcx_file.tcx, TCXParser)


def test_extract_track_points(tcx_file):
    """
    Test reading of TCX
    """
    tcx_file.read_tcx()
    tcx_file.extract_track_points()

    assert isinstance(tcx_file.track_points, zip)


def test_create_gpx(tcx_file):
    """
    Test reading of TCX
    """
    tcx_file.read_tcx()
    tcx_file.extract_track_points()
    tcx_file.create_gpx()

    assert isinstance(tcx_file.gpx, GPX)


def test_write_gpx(tcx_file):
    """
    Test reading of TCX
    """
    tcx_file.read_tcx()
    tcx_file.extract_track_points()
    tcx_file.create_gpx()
    tcx_file.write_gpx()

    assert Path(GPX_FILE).exists()


def test_convert(tcx_file):
    """
    Test conversion wrapper
    """
    tcx_file.convert()

    assert isinstance(tcx_file.track_points, zip)
    assert isinstance(tcx_file.gpx, GPX)



"""
Initialise the module.
"""
import logging
import sys

LOG_FORMATTER = logging.Formatter('%(asctime)s [%(levelname)s][%(name)s] %(message)s',
                                  datefmt="%Y-%m-%d %H:%MS")
LOG_ERR_FORMATTER = logging.Formatter(
    '%(asctime)s [%(levelname)s][%(name)s][%(filename)s:%(lineno)d] %(message)s',
    datefmt="%Y-%m-%d %H:%M:%S")
LOGGER_NAME = 'tcx2gpx'

STD_OUT_STREAMHANDLER = logging.StreamHandler(sys.stdout)
STD_OUT_STREAMHANDLER.setLevel(logging.DEBUG)
STD_OUT_STREAMHANDLER.setFormatter(LOG_FORMATTER)

STD_ERR_STREAM_HANDLER = logging.StreamHandler(sys.stderr)
STD_ERR_STREAM_HANDLER.setLevel(logging.ERROR)
STD_ERR_STREAM_HANDLER.setFormatter(LOG_ERR_FORMATTER)



"""
tcx2gpx setup
"""
import setuptools

with open('README.md', 'r') as fh:
    LONG_DESCRIPTION = fh.read()
setuptools.setup(use_scm_version=True,
                 description='Convert Garmin TPX to GPX',
                 long_description=LONG_DESCRIPTION,
                 long_description_content_type='text/markdown')
